package game;

import java.io.IOException;
import java.net.URL;

import javax.sound.midi.InvalidMidiDataException;
import javax.sound.midi.MetaEventListener;
import javax.sound.midi.MetaMessage;
import javax.sound.midi.MidiSystem;
import javax.sound.midi.MidiUnavailableException;
import javax.sound.midi.Receiver;
import javax.sound.midi.Sequence;
import javax.sound.midi.Sequencer;
import javax.sound.midi.Synthesizer;
import javax.sound.midi.Transmitter;

/*
 * Created on 2006/11/05
 *
 * 改変履歴
 * 2008/08/20 再生回数の設定を可能に変更
 *            登録名ではなく、インデックスで登録・取得を行うよう変更
 */

public class MidiEngine implements MetaEventListener {
    // 再生終了メタイベント
    private static final int END_OF_TRACK = 47;

    // シーケンサー
    private Sequencer sequencer;
    // シンセサイザー
    private Synthesizer synthesizer;

    // 登録できるMIDIファイルの最大数
    private int maxSequences;

    // MIDIファイルデータ
    private Sequence[] midiData;

    // 現在再生中のMIDIシーケンスID
    private int currentSequenceId = -1;

    // ループ回数
    private int loopCount = 0;

    /**
     * コンストラクタ
     */
    public MidiEngine() {
        this(256);
    }

    /**
     * コンストラクタ
     *
     * @param maxSequences
     *            登録できるMIDIファイルの最大数
     */
    public MidiEngine(int maxSequences) {
        this.maxSequences = maxSequences;
        midiData = new Sequence[this.maxSequences];

        // シーケンサーとシンセサイザーを初期化
        initSequencer();
    }

    private void initSequencer() {
        try {
            // シーケンサを開く
            sequencer = MidiSystem.getSequencer();
            sequencer.open();
            // メタイベントリスナーを登録
            sequencer.addMetaEventListener(this);
            // シーケンサとシンセサイザーの接続
            if (!(sequencer instanceof Synthesizer)) { // J2SE 5.0用
                // シンセサイザーを開く
                synthesizer = MidiSystem.getSynthesizer();
                synthesizer.open();
                Receiver synthReceiver = synthesizer.getReceiver();
                Transmitter seqTransmitter = sequencer.getTransmitter();
                seqTransmitter.setReceiver(synthReceiver);
            } else { // J2SE 1.4.2以前
                // シーケンサとシンセサイザーは同じ
                synthesizer = (Synthesizer) sequencer;
            }
        } catch (MidiUnavailableException e) {
            e.printStackTrace();
        }
    }

    /**
     * MIDIファイルをロード
     *
     * @param id       登録ID
     * @param filename ファイル名
     */
    public void load(int id, String filename) {
        try {
            // MIDIファイルをロード
            URL url = getClass().getClassLoader().getResource(filename);
            Sequence seq = MidiSystem.getSequence(url);
            // 登録
            midiData[id] = seq;

        } catch (InvalidMidiDataException e) {
            e.printStackTrace();
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    /**
     * 再生
     *
     * @param id 登録ID
     * @param loop ループ回数（n < 0:無限ループ）
     */
    public void play(int id, int loop) {

        // 現在再生中のシーケンスと同じなら何もしない
        if (currentSequenceId == id) {
            return;
        }

        // 現在再生中のシーケンスを停止する
        stop();

        // 名前に対応するMIDIを取得
        Sequence seq = midiData[id];
        if (sequencer != null && seq != null) {
            try {
                // シーケンサにセット
                sequencer.setSequence(seq);
                // 再生開始！
                sequencer.start();
                currentSequenceId = id;
                loopCount = loop;
            } catch (InvalidMidiDataException e) {
                e.printStackTrace();
            }
        }
    }

    /**
     * 停止
     */
    public void stop() {
        if (sequencer.isRunning()) {
            sequencer.stop();
        }
    }

    /**
     * 終了処理
     */
    public void close() {
        // 再生中のシーケンスを停止する
        stop();

        // シーケンサの終了処理
        sequencer.removeMetaEventListener(this);
        sequencer.close();
        sequencer = null;
    }

    public void meta(MetaMessage meta) {
        // 再生が終了した場合
        if (meta.getType() == END_OF_TRACK) {
            if (sequencer != null && sequencer.isOpen()) {

                if (loopCount > 0) --loopCount;

                if (loopCount != 0) {
                    // MIDIシーケンス再生位置に戻す
                    sequencer.setMicrosecondPosition(0);
                    // 最初から再生
                    sequencer.start();
                }
            }
        }
    }
}
